;PhysicEngine_3.au3
#include <Misc.au3>
#include <Timers.au3>
#include <PhysicEngine_3_Collision.au3>

Func phys_Startup()
	phys_LoadSettings()							
	phys_StartupDraw()							
	OnAutoItExitRegister( "phys_Shutdown" )
EndFunc

Func phys_Shutdown()
	phys_ShutdownDraw() 
	phys_SaveSettings()
EndFunc

Global $g_StartTime = _Timer_Init()
Global $g_TimerDiff_Last = _Timer_Diff($g_StartTime)
Global $g_Gravity[2] = [0,0]
Global $g_Coefficient_Damping = 0.99
Global $g_CorrectPositions = True
Global $g_Friction = True
Global $g_DrawInformations = False
Global $g_ImpulseScale = 0.9
Global $g_Sleep_MaxMovementSQ = 5*5
Global $g_Sleep_MaxAngle = 5
Global $g_Sleep_TimeTo = 3

Func phys_Frame()
	
	Local $TimerDiff = _Timer_Diff($g_StartTime)
	Local $DeltaT_s = 0.001 * ($TimerDiff - $g_TimerDiff_Last) 	
	$g_TimerDiff_Last = $TimerDiff
	
	WinSetTitle( $g_hWindow, "", $g_Window_Title & " - " & Round( $DeltaT_s * 1000, 0 ) & "ms " & Round( 1/$DeltaT_s, 0 ) & "fps" )
	
	phys_UserInput()
		
	phys_PolygonsUpdatePoints()
	
	For $i = 0 To $g_NumPolygons - 1 Step 1
		phys_PolygonAddForceCenter( $i, $g_Gravity[0], $g_Gravity[1] )
	Next	
	
	For $i = 0 To $g_NumCircles - 1 Step 1
		phys_CircleAddForceCenter( $i, $g_Gravity[0], $g_Gravity[1] )
	Next
	
	phys_PolygonsApplyForces( $DeltaT_s )
	phys_CirclesApplyForces( $DeltaT_s )
	
	phys_Collisions()
	
	phys_PolygonsUpdatePosition( $DeltaT_s, $g_Coefficient_Damping, $g_Sleep_MaxMovementSQ, $g_Sleep_MaxAngle, $g_Sleep_TimeTo )
	phys_CirclesUpdatePosition( $DeltaT_s, $g_Coefficient_Damping, $g_Sleep_MaxMovementSQ, $g_Sleep_MaxAngle, $g_Sleep_TimeTo )
	
	phys_PolygonsUpdatePoints()
	
	phys_Draw( $DeltaT_s )
		
	;ToolTip( "DeltaT: " & Round( $DeltaT_s * 1000, 0 ) & "ms -> " & Round( 1/$DeltaT_s, 0 ) & "fps" )
EndFunc

Global $g_Selected_Radius[2]
Global $g_Selected_ID = -1
Enum $OBJECT_POLYGON, $OBJECT_CIRCLE
Global Const $NON_SELECTED = -1
Global $g_Selected_Type = 0

Func phys_Draw( $delta_t_s )

	phys_ClearBackbuffer()	
	
	For $i = 0 To $g_NumPolygons - 1 Step 1
		phys_DrawPolygon( $i )
	Next
	
	For $i = 0 To $g_NumCircles - 1 Step 1
		phys_DrawCircle( $i )
	Next	
	
	Local $MousePosition = MouseGetPos()
	
	If $g_Selected_ID <> $NON_SELECTED Then
		If $g_Selected_Type = $OBJECT_POLYGON Then
			Local $Sin = Sin( $g_Polygons_Angle[$g_Selected_ID] )
			Local $Cos = Cos( $g_Polygons_Angle[$g_Selected_ID] )
			
			Local $ApplyPointX = $g_Polygons_Position_X[$g_Selected_ID] + $Cos * $g_Selected_Radius[0] - $sin * $g_Selected_Radius[1]
			Local $ApplyPointY = $g_Polygons_Position_Y[$g_Selected_ID] + $Sin * $g_Selected_Radius[0] + $Cos * $g_Selected_Radius[1] 
			
			_GDIPlus_GraphicsDrawLine($g_hBackbuffer,$ApplyPointX-5,$ApplyPointY,$ApplyPointX+5,$ApplyPointY, $g_hPen)
			_GDIPlus_GraphicsDrawLine($g_hBackbuffer,$ApplyPointX,$ApplyPointY-5,$ApplyPointX,$ApplyPointY+5, $g_hPen)
			
			_GDIPlus_GraphicsDrawLine($g_hBackbuffer,$g_Polygons_Position_X[$g_Selected_ID]-5,$g_Polygons_Position_Y[$g_Selected_ID],$g_Polygons_Position_X[$g_Selected_ID]+5,$g_Polygons_Position_Y[$g_Selected_ID], $g_hPen)
			_GDIPlus_GraphicsDrawLine($g_hBackbuffer,$g_Polygons_Position_X[$g_Selected_ID],$g_Polygons_Position_Y[$g_Selected_ID]-5,$g_Polygons_Position_X[$g_Selected_ID],$g_Polygons_Position_Y[$g_Selected_ID]+5, $g_hPen)
		
			If _IsPressed( 0x1 ) Then _GDIPlus_GraphicsDrawLine($g_hBackbuffer,$ApplyPointX,$ApplyPointY,$MousePosition[0],$MousePosition[1], $g_hPen)
		ElseIf $g_Selected_Type = $OBJECT_CIRCLE Then
			Local $Sin = Sin( $g_Circles_Angle[$g_Selected_ID] )
			Local $Cos = Cos( $g_Circles_Angle[$g_Selected_ID] )
			
			Local $ApplyPointX = $g_Circles_Position_X[$g_Selected_ID] + $Cos * $g_Selected_Radius[0] - $sin * $g_Selected_Radius[1]
			Local $ApplyPointY = $g_Circles_Position_Y[$g_Selected_ID] + $Sin * $g_Selected_Radius[0] + $Cos * $g_Selected_Radius[1] 
			
			_GDIPlus_GraphicsDrawLine($g_hBackbuffer,$ApplyPointX-5,$ApplyPointY,$ApplyPointX+5,$ApplyPointY, $g_hPen)
			_GDIPlus_GraphicsDrawLine($g_hBackbuffer,$ApplyPointX,$ApplyPointY-5,$ApplyPointX,$ApplyPointY+5, $g_hPen)
			
			_GDIPlus_GraphicsDrawLine($g_hBackbuffer,$g_Circles_Position_X[$g_Selected_ID]-5,$g_Circles_Position_Y[$g_Selected_ID],$g_Circles_Position_X[$g_Selected_ID]+5,$g_Circles_Position_Y[$g_Selected_ID], $g_hPen)
			_GDIPlus_GraphicsDrawLine($g_hBackbuffer,$g_Circles_Position_X[$g_Selected_ID],$g_Circles_Position_Y[$g_Selected_ID]-5,$g_Circles_Position_X[$g_Selected_ID],$g_Circles_Position_Y[$g_Selected_ID]+5, $g_hPen)
			
			If _IsPressed( 0x1 ) Then _GDIPlus_GraphicsDrawLine($g_hBackbuffer,$ApplyPointX,$ApplyPointY,$MousePosition[0],$MousePosition[1], $g_hPen)
		EndIf
	EndIf
	
	;_GDIPlus_GraphicsDrawLine($g_hBackbuffer,$g_point_2[0],$g_point_2[1],$g_point_2[0] + $g_mtd_2[0] * 20,$g_point_2[1] + $g_mtd_2[1] * 20, $g_hPen)

    _GDIPlus_GraphicsDrawLine($g_hBackbuffer,$MousePosition[0]-5,$MousePosition[1],$MousePosition[0]+5,$MousePosition[1], $g_hPen)
    _GDIPlus_GraphicsDrawLine($g_hBackbuffer,$MousePosition[0],$MousePosition[1]-5,$MousePosition[0],$MousePosition[1]+5, $g_hPen)
	
	_GDIPlus_GraphicsDrawLine($g_hBackbuffer,$depthest_point_x,$depthest_point_y,$depthest_point_x+$penetration_x,$depthest_point_y+$penetration_y, $g_hPen)
	
	phys_SwapBuffers()	
	
EndFunc

Global $penetration_x = 0
Global $penetration_y = 0
Global $penetration_depth = 0 
Global $depthest_point_x = 0
Global $depthest_point_y = 0

Global $g_DebugEnable = False
Global $g_DebugWindow[10]

Global Const $g_SettingsFile = @ScriptDir & "\phys_Settings.ini"

Func phys_Collisions()
	For $i = 0 To $g_NumPolygons - 1 Step 1
		For $j = $i + 1 To $g_NumPolygons - 1 Step 1
			If (($g_Polygons_Mass[$i] <> 0) Or ($g_Polygons_Mass[$j] <> 0) Or ($g_Polygons_Tensor[$i] <> 0) Or ($g_Polygons_Tensor[$j] <> 0)) And (Not $g_Polygons_Sleep[$i] Or Not $g_Polygons_Sleep[$j]) Then
			
				Local $point_x = 0
				Local $point_y = 0
				Local $normal_x = 0
				Local $normal_y = 0
				
				If phys_PolygonsIntersect( $i, $j, $point_x, $point_y, $normal_x, $normal_y ) Then
					;ToolTip("collision polygon-polygon! point: " & $point_x & " " & $point_y & " normal: " & $normal_x & " " & $normal_y)
					phys_ResponseCollisionPolygons( $i, $j, $point_x, $point_y, $normal_x, $normal_y )	

					phys_PolygonsPenetrationDepth( $i, $j, $normal_x, $normal_y, $penetration_x, $penetration_y, $penetration_depth, $depthest_point_x, $depthest_point_y )
					
					If $g_Friction Then
						phys_ResponseFrictionPolygon( $i, $point_x, $point_y, -$normal_x, -$normal_y )
						phys_ResponseFrictionPolygon( $j, $point_x, $point_y,  $normal_x,  $normal_y )
					EndIf
					
					$g_Polygons_Sleep[$i] = False
					$g_Polygons_Sleep[$j] = False
					
					If $g_CorrectPositions Then phys_PolygonsFixPositions( $i, $j, $penetration_x, $penetration_y )
				EndIf
			EndIf
		Next
	Next
	
	For $i = 0 To $g_NumPolygons - 1 Step 1
		For $j = 0 To $g_NumCircles - 1 Step 1
			If (($g_Polygons_Mass[$i] <> 0) Or ($g_Circles_Mass[$j] <> 0) Or ($g_Polygons_Tensor[$i] <> 0) Or ($g_Circles_Tensor[$j] <> 0)) And (Not $g_Polygons_Sleep[$i] Or Not $g_Circles_Sleep[$j]) Then
			
				Local $point_x = 0
				Local $point_y = 0
				Local $normal_x = 0
				Local $normal_y = 0
			
				If phys_PolygonCircleIntersect( $i, $j, $point_x, $point_y, $normal_x, $normal_y ) Then
					
					phys_ResponseCollisionPolygonCircle( $i, $j, $point_x, $point_y, $normal_x, $normal_y )
					
					phys_PolygonCirclePenetrationDepth( $i, $j, $normal_x, $normal_y, $penetration_x, $penetration_y, $penetration_depth, $depthest_point_x, $depthest_point_y )
					;ToolTip("collision polygon-circle! point: " & $point_x & " " & $point_y & " normal: " & $normal_x & " " & $normal_y & " penetration: " & $penetration_x & " " & $penetration_y)
					If $g_Friction Then
						phys_ResponseFrictionPolygon( $i, $point_x, $point_y, -$normal_x, -$normal_y )
						phys_ResponseFrictionCircle( $j, $point_x, $point_y,  $normal_x,  $normal_y )
					EndIf
					
					$g_Polygons_Sleep[$i] = False
					$g_Circles_Sleep[$j] = False
					
					If $g_CorrectPositions Then phys_PolygonCircleFixPositions( $i, $j, $penetration_x, $penetration_y )
				EndIf
			EndIf
		Next
	Next
	
	For $i = 0 To $g_NumCircles - 1 Step 1
		For $j = $i+1 To $g_NumCircles - 1 Step 1
			If (($g_Circles_Mass[$i] <> 0) Or ($g_Circles_Mass[$j] <> 0) Or ($g_Circles_Tensor[$i] <> 0) Or ($g_Circles_Tensor[$j] <> 0)) And (Not $g_Circles_Sleep[$i] Or Not $g_Circles_Sleep[$j]) Then
			
				Local $point_x = 0
				Local $point_y = 0
				Local $normal_x = 0
				Local $normal_y = 0
			
				If phys_CirclesIntersect( $i, $j, $point_x, $point_y, $normal_x, $normal_y ) Then
					;ToolTip("collision circle-circle! point: " & $point_x & " " & $point_y & " normal: " & $normal_x & " " & $normal_y)
					phys_ResponseCollisionCircles( $i, $j, $point_x, $point_y, $normal_x, $normal_y )
					
					phys_CirclesPenetrationDepth( $i, $j, $normal_x, $normal_y, $penetration_x, $penetration_y, $penetration_depth, $depthest_point_x, $depthest_point_y )
					
					If $g_Friction Then
						phys_ResponseFrictionCircle( $i, $point_x, $point_y, -$normal_x, -$normal_y )
						phys_ResponseFrictionCircle( $j, $point_x, $point_y,  $normal_x,  $normal_y )
					EndIf
					
					$g_Circles_Sleep[$i] = False
					$g_Circles_Sleep[$j] = False
					
					If $g_CorrectPositions Then phys_CirclesFixPositions( $i, $j, $penetration_x, $penetration_y )
				EndIf
			EndIf
		Next
	Next
EndFunc

Global $g_Mouse_Left_Last = False
Global $g_Key_C_Last = False

Func phys_UserInput()
	If Not $g_DebugEnable Then Return 0

	phys_ReadDebugWindow()
	
	Local $MousePosition = MouseGetPos()
	Local $MouseLeft = _IsPressed( 0x1 ) ; left mouse	
	Local $KeyC = _IsPressed( 43 ); c key
	
	If $MouseLeft And Not $g_Mouse_Left_Last Then
		$g_Selected_ID = $NON_SELECTED
		
		Local $Radius
		
		If phys_PolygonFromPoint( $MousePosition[0], $MousePosition[1], $g_Selected_ID ) Then
			$g_Selected_Type = $OBJECT_POLYGON
			$Radius = phys_PolygonGetRadiusPoint( $g_Selected_ID, $MousePosition[0], $MousePosition[1] )
			Local $Sin = Sin( -$g_Polygons_Angle[$g_Selected_ID] )
			Local $Cos = Cos( -$g_Polygons_Angle[$g_Selected_ID] )
						
			$g_Selected_Radius[0] = $Cos * $Radius[0] - $sin * $Radius[1]
			$g_Selected_Radius[1] = $Sin * $Radius[0] + $Cos * $Radius[1]
		ElseIf phys_CircleFromPoint( $MousePosition[0], $MousePosition[1], $g_Selected_ID ) Then
			$g_Selected_Type = $OBJECT_CIRCLE
			$Radius = phys_CircleGetRadiusPoint( $g_Selected_ID, $MousePosition[0], $MousePosition[1] )
			Local $Sin = Sin( -$g_Circles_Angle[$g_Selected_ID] )
			Local $Cos = Cos( -$g_Circles_Angle[$g_Selected_ID] )
						
			$g_Selected_Radius[0] = $Cos * $Radius[0] - $sin * $Radius[1]
			$g_Selected_Radius[1] = $Sin * $Radius[0] + $Cos * $Radius[1]
		EndIf
	EndIf
	
	If $g_Selected_ID <> $NON_SELECTED Then
		If $g_Selected_Type = $OBJECT_POLYGON Then	
			GUICtrlSetData( $g_DebugWindow[8], phys_PolygonGetInfomations($g_Selected_ID) )
				
			If $MouseLeft Then ; left mouse	 
				Local $Sin = Sin( $g_Polygons_Angle[$g_Selected_ID] )
				Local $Cos = Cos( $g_Polygons_Angle[$g_Selected_ID] )
				
				Local $ApplyPointX = $g_Polygons_Position_X[$g_Selected_ID] + $Cos * $g_Selected_Radius[0] - $sin * $g_Selected_Radius[1]
				Local $ApplyPointY = $g_Polygons_Position_Y[$g_Selected_ID] + $Sin * $g_Selected_Radius[0] + $Cos * $g_Selected_Radius[1] 
				
				Local $ForceX = 10 * ($MousePosition[0] - $ApplyPointX)
				Local $ForceY = 10 * ($MousePosition[1] - $ApplyPointY)
				
				phys_PolygonAddForcePoint( $g_Selected_ID, $ApplyPointX, $ApplyPointY, $ForceX, $ForceY )
			EndIf
			
			If _IsPressed( 0x2 ) Then ; right mouse
				$g_Polygons_Position_X[$g_Selected_ID] = $MousePosition[0]
				$g_Polygons_Position_Y[$g_Selected_ID] = $MousePosition[1]
			EndIf
			
			If _IsPressed( 0x34 ) Then ; r key
				phys_PolygonStop($g_Selected_ID)
			EndIf
			
			If _IsPressed( 41 ) Then ; left key
				$g_Polygons_Angle[$g_Selected_ID] -= $DEG_TO_RAD
			EndIf
			
			If _IsPressed( 44 ) Then ; right key
				$g_Polygons_Angle[$g_Selected_ID] += $DEG_TO_RAD
			EndIf
			
			If Not $g_Key_C_Last And $KeyC Then
				$g_Selected_ID = phys_CopyPolygon( $g_Selected_ID )
			EndIf	

			If _IsPressed( "4A" ) Then phys_PolygonScale( $g_Selected_ID, 0.99, 1 )	; j key
			If _IsPressed( "4C" ) Then phys_PolygonScale( $g_Selected_ID, 1.01, 1 )	; l key
			If _IsPressed( "49" ) Then phys_PolygonScale( $g_Selected_ID, 1, 0.99 )	; i key
			If _IsPressed( "4B" ) Then phys_PolygonScale( $g_Selected_ID, 1, 1.01 )	; k key
			
			If _IsPressed( "45" ) Then
				phys_PolygonDelete( $g_Selected_ID )
				$g_Selected_ID = -1
			EndIf
			If _IsPressed( "53" ) Then phys_PolygonSleep( $g_Selected_ID )			
			If _IsPressed( "57" ) Then phys_PolygonWake( $g_Selected_ID )	
		ElseIf $g_Selected_Type = $OBJECT_CIRCLE Then
			GUICtrlSetData( $g_DebugWindow[8], phys_CircleGetInfomations($g_Selected_ID) )
			
			If $MouseLeft Then ; left mouse	 
				Local $Sin = Sin( $g_Circles_Angle[$g_Selected_ID] )
				Local $Cos = Cos( $g_Circles_Angle[$g_Selected_ID] )
				
				Local $ApplyPointX = $g_Circles_Position_X[$g_Selected_ID] + $Cos * $g_Selected_Radius[0] - $sin * $g_Selected_Radius[1]
				Local $ApplyPointY = $g_Circles_Position_Y[$g_Selected_ID] + $Sin * $g_Selected_Radius[0] + $Cos * $g_Selected_Radius[1] 
				
				Local $ForceX = 10 * ($MousePosition[0] - $ApplyPointX)
				Local $ForceY = 10 * ($MousePosition[1] - $ApplyPointY)
				
				phys_CircleAddForcePoint( $g_Selected_ID, $ApplyPointX, $ApplyPointY, $ForceX, $ForceY )
			EndIf
			
			If _IsPressed( 0x2 ) Then ; right mouse
				$g_Circles_Position_X[$g_Selected_ID] = $MousePosition[0]
				$g_Circles_Position_Y[$g_Selected_ID] = $MousePosition[1]
			EndIf
			
			If _IsPressed( 0x34 ) Then ; r key
				phys_CircleStop($g_Selected_ID)
			EndIf
			If _IsPressed( "53" ) Then phys_CircleSleep( $g_Selected_ID )		
			If _IsPressed( "57" ) Then phys_CircleWake( $g_Selected_ID )
		EndIf
	EndIf
	
	$g_Mouse_Left_Last = $MouseLeft
	$g_Key_C_Last = $KeyC
EndFunc

Func phys_SetDamping( $damping )
	$g_Coefficient_Damping = $damping
EndFunc

Func phys_SetImpulseScale( $scale )
	$g_ImpulseScale = $scale
EndFunc

Func phys_SetGravity( $gravity_x, $gravity_y  )
	$g_Gravity[0] = $gravity_x
	$g_Gravity[1] = $gravity_y
EndFunc

Func phys_SetClearColor( $color )
	$g_ClearColor = $color
EndFunc

Func phys_CreateDebugWindow()
	$g_DebugEnable = True

	WinMove( $g_hWindow, "", $g_Window_X, $g_Window_Y, $g_Window_Width + 300, $g_Window_Height)
	
	Local $label_pos_x = $g_Window_Width + 20
	Local $label_pos_y = 20
	Local $edit_pos_x = $g_Window_Width + 110
	Local $edit_pos_y = 20
	Local $label_width = 80
	Local $label_height = 25
	Local $edit_width = 80
	Local $edit_height = 20
	Local $object_offset_y = 30
	Local $checkbox_width = 20
	Local $checkbox_height = 20
	Local $info_width = 280
	Local $info_height = 110
	Local $info_2_height = 250
	Local $offset_infos_y = 0
		
	Local Const $ES_AUTOHSCROLL = 0x0040
	Local Const $ES_AUTOVSCROLL = 0x0080
	
	GUICtrlCreateLabel( "Damping:", $label_pos_x, $label_pos_y, $label_width, $label_height )
	$g_DebugWindow[0] = GUICtrlCreateEdit( $g_Coefficient_Damping, $edit_pos_x, $edit_pos_y, $edit_width, $edit_height, BitOr($ES_AUTOVSCROLL, $ES_AUTOHSCROLL))
		$label_pos_y += $object_offset_y
		$edit_pos_y += $object_offset_y
		
	GUICtrlCreateLabel( "ImpulseScale:", $label_pos_x, $label_pos_y, $label_width, $label_height )
	$g_DebugWindow[1] = GUICtrlCreateEdit( $g_ImpulseScale, $edit_pos_x, $edit_pos_y, $edit_width, $edit_height, BitOr($ES_AUTOVSCROLL, $ES_AUTOHSCROLL))
		$label_pos_y += $object_offset_y
		$edit_pos_y += $object_offset_y
		
	GUICtrlCreateLabel( "Gravity:", $label_pos_x, $label_pos_y, $label_width, $label_height )
	$g_DebugWindow[2] = GUICtrlCreateEdit( $g_Gravity[0], $edit_pos_x, $edit_pos_y, $edit_width, $edit_height, BitOr($ES_AUTOVSCROLL, $ES_AUTOHSCROLL))
	$g_DebugWindow[3] = GUICtrlCreateEdit( $g_Gravity[1], $edit_pos_x + 100, $edit_pos_y, $edit_width, $edit_height, BitOr($ES_AUTOVSCROLL, $ES_AUTOHSCROLL))
		$label_pos_y += $object_offset_y
		$edit_pos_y += $object_offset_y
			
	GUICtrlCreateLabel( "ClearColor:", $label_pos_x, $label_pos_y, $label_width, $label_height )
	$g_DebugWindow[4] = GUICtrlCreateEdit( "0x" & Hex( $g_ClearColor ), $edit_pos_x, $edit_pos_y, $edit_width, $edit_height, BitOr($ES_AUTOVSCROLL, $ES_AUTOHSCROLL))
		$label_pos_y += $object_offset_y
		$edit_pos_y += $object_offset_y
	
	Local Const $GUI_CHECKED = 1
	Local Const $GUI_UNCHECKED = 4
	
	GUICtrlCreateLabel( "CorrectPositions:", $label_pos_x, $label_pos_y, $label_width, $label_height )
	$g_DebugWindow[5] = GUICtrlCreateCheckbox( "", $edit_pos_x, $edit_pos_y, $checkbox_width, $checkbox_height ) 
		If $g_CorrectPositions Then GUICtrlSetState ( $g_DebugWindow[5], $GUI_CHECKED )
		If Not $g_CorrectPositions Then GUICtrlSetState ( $g_DebugWindow[5], $GUI_UNCHECKED )
		$label_pos_y += $object_offset_y
		$edit_pos_y += $object_offset_y
	
	GUICtrlCreateLabel( "DrawInformations:", $label_pos_x, $label_pos_y, $label_width, $label_height )
	$g_DebugWindow[6] = GUICtrlCreateCheckbox( "", $edit_pos_x, $edit_pos_y, $checkbox_width, $checkbox_height ) 
		If $g_DrawInformations Then GUICtrlSetState ( $g_DebugWindow[6], $GUI_CHECKED )
		If Not $g_DrawInformations Then GUICtrlSetState ( $g_DebugWindow[6], $GUI_UNCHECKED )
		$label_pos_y += $object_offset_y
		$edit_pos_y += $object_offset_y		
	
	GUICtrlCreateLabel( "Friction:", $label_pos_x, $label_pos_y, $label_width, $label_height )
	$g_DebugWindow[7] = GUICtrlCreateCheckbox( "", $edit_pos_x, $edit_pos_y, $checkbox_width, $checkbox_height ) 
		If $g_Friction Then GUICtrlSetState ( $g_DebugWindow[7], $GUI_CHECKED )
		If Not $g_Friction Then GUICtrlSetState ( $g_DebugWindow[7], $GUI_UNCHECKED )
		$label_pos_y += $object_offset_y
		$edit_pos_y += $object_offset_y	
			
	$g_DebugWindow[8] = GUICtrlCreateLabel( "Informations:", $label_pos_x, $label_pos_y, $info_width, $info_height )
		$label_pos_y += $offset_infos_y + $info_height
		$edit_pos_y += $offset_infos_y + $info_height
		
	Local $Info_Key = _
		"Controls: " & @LF & @LF & _ 
		"   + 'W' = Wake Body" & @LF & _ 
		"   + 'S' = Sleep Body" & @LF & _ 
		"   + 'A' = Rotate Body Clockwise" & @LF & _ 
		"   + 'D' = Rotate Body Anticlockwise" & @LF & _ 
		"   + 'K' = Increase Body Size X" & @LF & _ 
		"   + 'I' = Decrease Body Size X" & @LF & _ 
		"   + 'L' = Increase Body Size Y" & @LF & _ 
		"   + 'J' = Decrease Body Size Y" & @LF & _ 
		"   + 'R' = Reset Body Velocity" & @LF & _ 
		"   + 'E' = Erase Body" & @LF & _ 
		"   + 'Mouse Lelf' = Select/Accelerate Body" & @LF & _ 
		"   + 'Mouse Right' = Set Body Position" & @LF & _ 
		"   + 'Left' = Player Move Left" & @LF & _ 
		"   + 'Right' = Player Move Right" & @LF & _ 
		"   + 'Up' = Player Jump"
		
	GUICtrlCreateLabel( $Info_Key, $label_pos_x, $label_pos_y, $info_width, $info_2_height )
		$label_pos_y += $object_offset_y
		$edit_pos_y += $object_offset_y	
EndFunc

Func phys_ReadDebugWindow()
	$g_Coefficient_Damping = GUICtrlRead( $g_DebugWindow[0] )
	$g_ImpulseScale = GUICtrlRead( $g_DebugWindow[1] )
	$g_Gravity[0] = GUICtrlRead( $g_DebugWindow[2] )
	$g_Gravity[1] = GUICtrlRead( $g_DebugWindow[3] )
	$g_ClearColor = "0x" & Hex( GUICtrlRead( $g_DebugWindow[4] ), 8 )
	Local Const $GUI_CHECKED = 1
	Local Const $GUI_UNCHECKED = 4
	$g_CorrectPositions = (GUICtrlRead( $g_DebugWindow[5] ) = $GUI_CHECKED)
	$g_DrawInformations = (GUICtrlRead( $g_DebugWindow[6] ) = $GUI_CHECKED)
	$g_Friction = (GUICtrlRead( $g_DebugWindow[7] ) = $GUI_CHECKED)
		
	If MouseGetPos(0) > $g_Window_Width Then 
		GUISetCursor(-1, 1, $g_hWindow)
	Else
		GUISetCursor(16, 1, $g_hWindow)
	EndIf
EndFunc

Func phys_SaveSettings()
	IniWrite( $g_SettingsFile, "Settings", "Damping", $g_Coefficient_Damping )
	IniWrite( $g_SettingsFile, "Settings", "ImpulseScale", $g_ImpulseScale )
	IniWrite( $g_SettingsFile, "Settings", "GravityX", $g_Gravity[0] )
	IniWrite( $g_SettingsFile, "Settings", "GravityY", $g_Gravity[1] )
	IniWrite( $g_SettingsFile, "Settings", "ClearColor", $g_ClearColor )
	If Not $g_CorrectPositions Then IniWrite( $g_SettingsFile, "Settings", "CorrectPositions", 0 )
	If $g_CorrectPositions Then IniWrite( $g_SettingsFile, "Settings", "CorrectPositions", 1 )
	If Not $g_DrawInformations Then IniWrite( $g_SettingsFile, "Settings", "DrawInformations", 0 )
	If $g_DrawInformations Then IniWrite( $g_SettingsFile, "Settings", "DrawInformations", 1 )
	If Not $g_Friction Then IniWrite( $g_SettingsFile, "Settings", "Friction", 0 )
	If $g_Friction Then IniWrite( $g_SettingsFile, "Settings", "Friction", 1 )
EndFunc

Func phys_LoadSettings()
	$g_Coefficient_Damping = IniRead( $g_SettingsFile, "Settings", "Damping", 0.99 )
	$g_ImpulseScale = IniRead( $g_SettingsFile, "Settings", "ImpulseScale", 0.6 )
	$g_Gravity[0] = IniRead( $g_SettingsFile, "Settings", "GravityX", 0 )
	$g_Gravity[1] = IniRead( $g_SettingsFile, "Settings", "GravityY", 98.1 )
	$g_ClearColor = IniRead( $g_SettingsFile, "Settings", "ClearColor", 0xFF819FF7 )
	$g_CorrectPositions = (IniRead( $g_SettingsFile, "Settings", "CorrectPositions", 1 ) = 1)
	$g_DrawInformations = (IniRead( $g_SettingsFile, "Settings", "DrawInformations", 0 ) = 1)
	$g_Friction = (IniRead( $g_SettingsFile, "Settings", "Friction", 1 ) = 1)
EndFunc