;PhysicEngine_3_Collision.au3

#include <PhysicEngine_3_Draw.au3>

Func phys_PolygonsIntersect( $polygon_id_1, $polygon_id_2, ByRef $point_x, ByRef $point_y, ByRef $normal_x, ByRef $normal_y )
	
	Local $DeltaX = $g_Polygons_Position_X[$polygon_id_2] - $g_Polygons_Position_X[$polygon_id_1]
	Local $DeltaY = $g_Polygons_Position_Y[$polygon_id_2] - $g_Polygons_Position_Y[$polygon_id_1]
	Local $Distance = Sqrt($DeltaX*$DeltaX + $DeltaY*$DeltaY)
	
	If $Distance > ($g_Polygons_Radius[$polygon_id_1] + $g_Polygons_Radius[$polygon_id_2]) Then Return False
	
	Local $NumIntersectionPoints = 0
	$point_x = 0
	$point_y = 0
	
	Local $NumElements1 = 2 * $g_Polygons_NumPoints[$polygon_id_1]
	Local $NumElements2 = 2 * $g_Polygons_NumPoints[$polygon_id_2]
	
	Local $SideNormalAlreadyAdded2[$NumElements2]
	For $i = 0 To $NumElements2 - 1 Step 1
		$SideNormalAlreadyAdded2[$i] = False
	Next
	
	Local $Normals1[3]
	Local $Normals2[3]
	
	For $i = 0 To $NumElements1 - 1 Step 2		
		Local $ii = Mod( $i+2, $NumElements1 )		
		
		Local $SideNormalAlreadyAdded1 = False
		
		For $j = 0 To $NumElements2 - 1 Step 2		
			Local $jj = Mod( $j+2, $NumElements2 )
			
			Local $InterectionPoint[2]
			
			If phys_LinesIntersect( _ 
				$g_Polygons_Points_Translated[$polygon_id_1][$i ], $g_Polygons_Points_Translated[$polygon_id_1][$i +1], _ 
				$g_Polygons_Points_Translated[$polygon_id_1][$ii], $g_Polygons_Points_Translated[$polygon_id_1][$ii+1], _ 
				$g_Polygons_Points_Translated[$polygon_id_2][$j ], $g_Polygons_Points_Translated[$polygon_id_2][$j +1], _ 
				$g_Polygons_Points_Translated[$polygon_id_2][$jj], $g_Polygons_Points_Translated[$polygon_id_2][$jj+1], _ 
				$InterectionPoint ) Then				
				
				$point_x += $InterectionPoint[0]
				$point_y += $InterectionPoint[1]
				
				Local $OrthogonalX
				Local $OrthogonalY
				
				If Not $SideNormalAlreadyAdded1 Then	
					$SideNormalAlreadyAdded1 = True
					phys_LineOrthogonal( _ 
						$g_Polygons_Points_Translated[$polygon_id_1][$i ], $g_Polygons_Points_Translated[$polygon_id_1][$i +1], _ 
						$g_Polygons_Points_Translated[$polygon_id_1][$ii], $g_Polygons_Points_Translated[$polygon_id_1][$ii+1], _
						$OrthogonalX, $OrthogonalY )
					
					$Normals1[0] -= $OrthogonalX
					$Normals1[1] -= $OrthogonalY
					$Normals1[2] += 1
				EndIf
				
				If Not $SideNormalAlreadyAdded2[$j] Then
					$SideNormalAlreadyAdded2[$j] = True
					phys_LineOrthogonal( _ 
						$g_Polygons_Points_Translated[$polygon_id_2][$j ], $g_Polygons_Points_Translated[$polygon_id_2][$j +1], _ 
						$g_Polygons_Points_Translated[$polygon_id_2][$jj], $g_Polygons_Points_Translated[$polygon_id_2][$jj+1], _
						$OrthogonalX, $OrthogonalY )
					
					$Normals2[0] += $OrthogonalX
					$Normals2[1] += $OrthogonalY
					$Normals2[2] += 1
				EndIf
				
				$NumIntersectionPoints += 1
			EndIf				
		Next
	Next
	
	If $NumIntersectionPoints = 0 Then 
		If $NumElements1 <> 0 Then
			If phys_IsPointInPolygon( $polygon_id_2, $g_Polygons_Points_Translated[$polygon_id_1][0], $g_Polygons_Points_Translated[$polygon_id_1][1] ) Then
				$normal_x = -$DeltaX
				$normal_y = -$DeltaY			
				phys_vec_normalize( $normal_x, $normal_y )	
				$point_x = $g_Polygons_Position_X[$polygon_id_1]
				$point_y = $g_Polygons_Position_Y[$polygon_id_1]
				Return True
			EndIf
		EndIf
		If $NumElements2 <> 0 Then
			If phys_IsPointInPolygon( $polygon_id_1, $g_Polygons_Points_Translated[$polygon_id_2][0], $g_Polygons_Points_Translated[$polygon_id_2][1] ) Then
				$normal_x = -$DeltaX
				$normal_y = -$DeltaY
				phys_vec_normalize( $normal_x, $normal_y )
				$point_x = $g_Polygons_Position_X[$polygon_id_2]
				$point_y = $g_Polygons_Position_Y[$polygon_id_2]
				Return True
			EndIf
		EndIf	
	
		Return False	
	EndIf
	
	$point_x /= $NumIntersectionPoints
	$point_y /= $NumIntersectionPoints
	
	If $Normals1[2] < $Normals2[2] Then
		$normal_x = $Normals1[0]
		$normal_y = $Normals1[1]
	Else
		$normal_x = $Normals2[0]
		$normal_y = $Normals2[1]
	EndIf
	
	If phys_vec_iszero( $normal_x, $normal_y ) Then
		$normal_x = -$DeltaX
		$normal_y = -$DeltaY
	EndIf
	
	phys_vec_normalize( $normal_x, $normal_y )
	
	Return True
EndFunc

Func phys_PolygonCircleIntersect( $polygon_id, $circle_id, ByRef $point_x, ByRef $point_y, ByRef $normal_x, ByRef $normal_y )
	
	Local $DeltaX = $g_Circles_Position_X[$circle_id] - $g_Polygons_Position_X[$polygon_id]
	Local $DeltaY = $g_Circles_Position_Y[$circle_id] - $g_Polygons_Position_Y[$polygon_id]
	Local $Distance = Sqrt($DeltaX*$DeltaX + $DeltaY*$DeltaY)
	
	If $Distance > ($g_Polygons_Radius[$polygon_id] + $g_Polygons_Radius[$circle_id]) Then Return False
	
	Local $NumIntersectionPoints = 0
	$point_x = 0
	$point_y = 0
	
	Local $NumElements1 = 2 * $g_Polygons_NumPoints[$polygon_id]
	
	For $i = 0 To $NumElements1 - 1 Step 2		
		Local $ii = Mod( $i+2, $NumElements1 )
	
		Local $NearestPoint = phys_LineNearestPoint( _ 
			$g_Polygons_Points_Translated[$polygon_id][$i ], $g_Polygons_Points_Translated[$polygon_id][$i +1], _ 
			$g_Polygons_Points_Translated[$polygon_id][$ii], $g_Polygons_Points_Translated[$polygon_id][$ii+1], _ 
			$g_Circles_Position_X[$circle_id], $g_Circles_Position_Y[$circle_id] )
			
		Local $Distance2 = phys_vec_distance( $NearestPoint[0], $NearestPoint[1], $g_Circles_Position_X[$circle_id], $g_Circles_Position_Y[$circle_id] )
		
		If $Distance2 < $g_Circles_Radius[$circle_id] Then 
			$NumIntersectionPoints += 1
			$point_x += $NearestPoint[0]
			$point_y += $NearestPoint[1]
		EndIf
	Next
	
	If $NumIntersectionPoints = 0 Then 
		If phys_IsPointInPolygon( $polygon_id, $g_Circles_Position_X[$circle_id], $g_Circles_Position_Y[$circle_id] ) Then
			$normal_x = -$DeltaX
			$normal_y = -$DeltaY			
			phys_vec_normalize( $normal_x, $normal_y )	
			$point_x = $g_Circles_Position_X[$circle_id]
			$point_y = $g_Circles_Position_Y[$circle_id]
			Return True
		EndIf		
		Return False
	EndIf
	
	$point_x /= $NumIntersectionPoints
	$point_y /= $NumIntersectionPoints
	
	$normal_x = $point_x - $g_Circles_Position_X[$circle_id]
	$normal_y = $point_y - $g_Circles_Position_Y[$circle_id]	
	
	phys_vec_normalize( $normal_x, $normal_y )
	
	If phys_IsPointInPolygon( $polygon_id, $g_Circles_Position_X[$circle_id], $g_Circles_Position_Y[$circle_id] ) Then
		$normal_x = -$normal_x
		$normal_y = -$normal_y
	EndIf
	
	Return True
EndFunc

Func phys_CirclesIntersect( $circle_id_1, $circle_id_2, ByRef $point_x, ByRef $point_y, ByRef $normal_x, ByRef $normal_y )
	
	Local $DeltaX = $g_Circles_Position_X[$circle_id_2] - $g_Circles_Position_X[$circle_id_1]
	Local $DeltaY = $g_Circles_Position_Y[$circle_id_2] - $g_Circles_Position_Y[$circle_id_1]
	Local $Distance = phys_vec_length($DeltaX, $DeltaY)
	
	Local $RadiusSum = $g_Circles_Radius[$circle_id_1] + $g_Circles_Radius[$circle_id_2]
	
	If $Distance > $RadiusSum Then Return False
	
	$normal_x = -$DeltaX
	$normal_y = -$DeltaY
	
	phys_vec_normalize( $normal_x, $normal_y )
	
	Local $Lambda = $g_Circles_Radius[$circle_id_1] / $RadiusSum
	
	$point_x = $g_Circles_Position_X[$circle_id_1] + $DeltaX * $Lambda
	$point_y = $g_Circles_Position_Y[$circle_id_1] + $DeltaY * $Lambda
	
	Return True
EndFunc

Func phys_PolygonsPenetrationDepth( $polygon_id_1, $polygon_id_2, $correction_dir_x, $correction_dir_y, ByRef $penetration_x, ByRef $penetration_y, ByRef $penetration_depth, ByRef $depthest_point_x, ByRef $depthest_point_y )
	
	;Local $vec_1_2_x = $correction_dir_x; $g_Polygons_Position_X[$polygon_id_2] - $g_Polygons_Position_X[$polygon_id_1]
	;Local $vec_1_2_y = $correction_dir_y; $g_Polygons_Position_Y[$polygon_id_2] - $g_Polygons_Position_Y[$polygon_id_1]
	
	phys_vec_setlength( $correction_dir_x, $correction_dir_y, 10000 )
	
	$penetration_x = 0
	$penetration_y = 0
	$penetration_depth = 0
	
	Local Const $NumElements1 = 2 * $g_Polygons_NumPoints[$polygon_id_1]
	Local Const $NumElements2 = 2 * $g_Polygons_NumPoints[$polygon_id_2]
	
	For $i = 0 To $NumElements1 - 1 Step 2 ; loop all points of polygon 2
				
		Local $NumIntersectionPoints = 0
		
		Local $NewPenetrationDepth[2] = [0,0]
		Local $DepthestPoint[2] = [0,0]
		Local $NewDepth = 0		
		
		Local $RayStartX = $g_Polygons_Points_Translated[$polygon_id_1][$i ]
		Local $RayStartY = $g_Polygons_Points_Translated[$polygon_id_1][$i+1]
		Local $RayEndX = $RayStartX + $correction_dir_x
		Local $RayEndY = $RayStartY + $correction_dir_y
		
		For $j = 0 To $NumElements2 - 1 Step 2			
			Local $jj = Mod( $j+2, $NumElements2 )
			
			Local $IntersectionPoint[2]
			
			If phys_LinesIntersect( $RayStartX, $RayStartY, $RayEndX, $RayEndY, _ 
				$g_Polygons_Points_Translated[$polygon_id_2][$j ], $g_Polygons_Points_Translated[$polygon_id_2][$j +1], _ 
				$g_Polygons_Points_Translated[$polygon_id_2][$jj], $g_Polygons_Points_Translated[$polygon_id_2][$jj+1], _ 
				$IntersectionPoint ) Then
				
				Local $NewPenetrationDepthX = $IntersectionPoint[0] - $RayStartX
				Local $NewPenetrationDepthY = $IntersectionPoint[1] - $RayStartY
				Local $NewDepth2 = phys_vec_length($NewPenetrationDepthX, $NewPenetrationDepthY) 
				
				If $NewDepth2 > $NewDepth Then	
					$NewDepth = $NewDepth2
					$NewPenetrationDepth[0] = $NewPenetrationDepthX
					$NewPenetrationDepth[1] = $NewPenetrationDepthY
					$DepthestPoint[0] = $IntersectionPoint[0]
					$DepthestPoint[1] = $IntersectionPoint[1]
				EndIf
				
				$NumIntersectionPoints += 1			
			EndIf		
		Next
		
		If Mod( $NumIntersectionPoints, 2 ) = 1 Then ; inside
			If $NewDepth > $penetration_depth Then
				$penetration_depth = $NewDepth
				$penetration_x = $NewPenetrationDepth[0]
				$penetration_y = $NewPenetrationDepth[1]
				$depthest_point_x = $DepthestPoint[0]
				$depthest_point_y = $DepthestPoint[1]
			EndIf
		EndIf
	Next
	
	For $i = 0 To $NumElements2 - 1 Step 2 ; loop all points of polygon 1
				
		Local $NumIntersectionPoints = 0
		
		Local $NewPenetrationDepth[2] = [0,0]
		Local $DepthestPoint[2] = [0,0]
		Local $NewDepth = 0		
		
		Local $RayStartX = $g_Polygons_Points_Translated[$polygon_id_2][$i ]
		Local $RayStartY = $g_Polygons_Points_Translated[$polygon_id_2][$i+1]
		Local $RayEndX = $RayStartX - $correction_dir_x
		Local $RayEndY = $RayStartY - $correction_dir_y
		
		For $j = 0 To $NumElements1 - 1 Step 2			
			Local $jj = Mod( $j+2, $NumElements1 )
			
			Local $IntersectionPoint[2]
			
			If phys_LinesIntersect( $RayStartX, $RayStartY, $RayEndX, $RayEndY, _ 
				$g_Polygons_Points_Translated[$polygon_id_1][$j ], $g_Polygons_Points_Translated[$polygon_id_1][$j +1], _ 
				$g_Polygons_Points_Translated[$polygon_id_1][$jj], $g_Polygons_Points_Translated[$polygon_id_1][$jj+1], _ 
				$IntersectionPoint ) Then
				
				Local $NewPenetrationDepthX = $RayStartX - $IntersectionPoint[0]
				Local $NewPenetrationDepthY = $RayStartY - $IntersectionPoint[1]
				Local $NewDepth2 = phys_vec_length($NewPenetrationDepthX, $NewPenetrationDepthY) 
				
				If $NewDepth2 > $NewDepth Then	
					$NewDepth = $NewDepth2
					$NewPenetrationDepth[0] = $NewPenetrationDepthX
					$NewPenetrationDepth[1] = $NewPenetrationDepthY
					$DepthestPoint[0] = $IntersectionPoint[0]
					$DepthestPoint[1] = $IntersectionPoint[1]
				EndIf
				
				$NumIntersectionPoints += 1			
			EndIf		
		Next
		
		If Mod( $NumIntersectionPoints, 2 ) = 1 Then ; inside
			If $NewDepth > $penetration_depth Then
				$penetration_depth = $NewDepth
				$penetration_x = $NewPenetrationDepth[0]
				$penetration_y = $NewPenetrationDepth[1]
				$depthest_point_x = $DepthestPoint[0]
				$depthest_point_y = $DepthestPoint[1]
			EndIf
		EndIf
	Next
	
EndFunc

Func phys_PolygonCirclePenetrationDepth( $polygon_id, $circle_id, $correction_dir_x, $correction_dir_y, ByRef $penetration_x, ByRef $penetration_y, ByRef $penetration_depth, ByRef $depthest_point_x, ByRef $depthest_point_y )
		
	$penetration_x = 0
	$penetration_y = 0
	$penetration_depth = 0
		
	phys_vec_setlength( $correction_dir_x, $correction_dir_y, -$g_Circles_Radius[$circle_id] )
				
	Local $RayStartX = $g_Circles_Position_X[$circle_id] - $correction_dir_x
	Local $RayStartY = $g_Circles_Position_Y[$circle_id] - $correction_dir_y
				
	phys_vec_setlength( $correction_dir_x, $correction_dir_y, 10000 )
				
	Local Const $NumElements1 = 2 * $g_Polygons_NumPoints[$polygon_id]
	
	For $i = 0 To $NumElements1 - 1 Step 2
		Local $ii = Mod( $i+2, $NumElements1 )
		
		Local $IntersectionPoint[2]
		
		If phys_LinesIntersect( $g_Polygons_Points_Translated[$polygon_id][$i  ], $g_Polygons_Points_Translated[$polygon_id][$i+1], _ 
			$g_Polygons_Points_Translated[$polygon_id][$ii  ], $g_Polygons_Points_Translated[$polygon_id][$ii+1], _
			$RayStartX, $RayStartY, $RayStartX + $correction_dir_x, $RayStartY + $correction_dir_y, $IntersectionPoint ) Then
			
			Local $OrthogoanlX = 0
			Local $OrthogoanlY = 0
			
			phys_LineOrthogonal( $g_Polygons_Points_Translated[$polygon_id][$i  ], $g_Polygons_Points_Translated[$polygon_id][$i+1], _ 
				$g_Polygons_Points_Translated[$polygon_id][$ii  ], $g_Polygons_Points_Translated[$polygon_id][$ii+1], _
				$OrthogoanlX, $OrthogoanlY )
				
			$OrthogoanlX *= $g_Circles_Radius[$circle_id]
			$OrthogoanlY *= $g_Circles_Radius[$circle_id]	
			
			Local $CenterIntersectionPointX = $IntersectionPoint[0] - $RayStartX
			Local $CenterIntersectionPointY = $IntersectionPoint[1] - $RayStartY
			
			$penetration_x = $IntersectionPoint[0] + $OrthogoanlX - $g_Circles_Position_X[$circle_id]
			$penetration_y = $IntersectionPoint[1] + $OrthogoanlY - $g_Circles_Position_Y[$circle_id]
			$penetration_x = -$penetration_x
			$penetration_y = -$penetration_y
			$penetration_depth = phys_vec_length( $penetration_x, $penetration_y )
			$depthest_point_x = $RayStartX
			$depthest_point_y = $RayStartY
			
			Return True
		EndIf		
	Next
	
	Return False
EndFunc

Func phys_CirclesPenetrationDepth( $circle_id_1, $circle_id_2, $correction_dir_x, $correction_dir_y, ByRef $penetration_x, ByRef $penetration_y, ByRef $penetration_depth, ByRef $depthest_point_x, ByRef $depthest_point_y )
		
	Local $DeltaX = $g_Circles_Position_X[$circle_id_1] - $g_Circles_Position_X[$circle_id_2]
	Local $DeltaY = $g_Circles_Position_Y[$circle_id_1] - $g_Circles_Position_Y[$circle_id_2]
	Local $Distance = phys_vec_length($DeltaX, $DeltaY)
		
	Local $RadiusSum = $g_Circles_Radius[$circle_id_1] + $g_Circles_Radius[$circle_id_2]
	
	$penetration_depth = $RadiusSum - $Distance
	
	$penetration_x = $DeltaX
	$penetration_y = $DeltaY
	
	phys_vec_setlength( $penetration_x, $penetration_y, $penetration_depth )
	phys_vec_setlength( $DeltaX, $DeltaY, $g_Circles_Radius[$circle_id_1] )
	
	$depthest_point_x = $g_Circles_Position_X[$circle_id_1] + $DeltaX
	$depthest_point_y = $g_Circles_Position_Y[$circle_id_1] + $DeltaY
EndFunc

Func phys_PolygonsFixPositions( $polygon_id_1, $polygon_id_2, $penetration_x, $penetration_y )

	If ($g_Polygons_Mass[$polygon_id_1] <> 0) And ($g_Polygons_Mass[$polygon_id_2] <> 0) Then
		$g_Polygons_Position_X[$polygon_id_1] += 0.5 * $penetration_x
		$g_Polygons_Position_Y[$polygon_id_1] += 0.5 * $penetration_y
		$g_Polygons_Position_X[$polygon_id_2] -= 0.5 * $penetration_x
		$g_Polygons_Position_Y[$polygon_id_2] -= 0.5 * $penetration_y
	ElseIf ($g_Polygons_Mass[$polygon_id_1] = 0) And ($g_Polygons_Mass[$polygon_id_2] = 0) Then	
	ElseIf $g_Polygons_Mass[$polygon_id_1] = 0 Then
		$g_Polygons_Position_X[$polygon_id_2] -= $penetration_x
		$g_Polygons_Position_Y[$polygon_id_2] -= $penetration_y
	Else
		$g_Polygons_Position_X[$polygon_id_1] += $penetration_x
		$g_Polygons_Position_Y[$polygon_id_1] += $penetration_y
	EndIf
	
EndFunc

Func phys_PolygonCircleFixPositions( $polygon_id, $circle_id, $penetration_x, $penetration_y )

	If ($g_Polygons_Mass[$polygon_id] <> 0) And ($g_Circles_Mass[$circle_id] <> 0) Then
		$g_Polygons_Position_X[$polygon_id] += 0.5 * $penetration_x
		$g_Polygons_Position_Y[$polygon_id] += 0.5 * $penetration_y
		$g_Circles_Position_X[$circle_id] -= 0.5 * $penetration_x
		$g_Circles_Position_Y[$circle_id] -= 0.5 * $penetration_y
	ElseIf ($g_Polygons_Mass[$polygon_id] = 0) And ($g_Circles_Mass[$circle_id] = 0) Then	
	ElseIf $g_Polygons_Mass[$polygon_id] = 0 Then
		$g_Circles_Position_X[$circle_id] -= $penetration_x
		$g_Circles_Position_Y[$circle_id] -= $penetration_y
	Else
		$g_Polygons_Position_X[$polygon_id] += $penetration_x
		$g_Polygons_Position_Y[$polygon_id] += $penetration_y
	EndIf
	
EndFunc

Func phys_CirclesFixPositions( $circle_id_1, $circle_id_2, $penetration_x, $penetration_y )

	If ($g_Polygons_Mass[$circle_id_1] <> 0) And ($g_Circles_Mass[$circle_id_2] <> 0) Then
		$g_Circles_Position_X[$circle_id_1] += 0.5 * $penetration_x
		$g_Circles_Position_Y[$circle_id_1] += 0.5 * $penetration_y
		$g_Circles_Position_X[$circle_id_2] -= 0.5 * $penetration_x
		$g_Circles_Position_Y[$circle_id_2] -= 0.5 * $penetration_y
	ElseIf ($g_Polygons_Mass[$circle_id_1] = 0) And ($g_Polygons_Mass[$circle_id_2] = 0) Then	
	ElseIf $g_Polygons_Mass[$circle_id_1] = 0 Then
		$g_Circles_Position_X[$circle_id_2] -= $penetration_x
		$g_Circles_Position_Y[$circle_id_2] -= $penetration_y
	Else
		$g_Circles_Position_X[$circle_id_1] += $penetration_x
		$g_Circles_Position_Y[$circle_id_1] += $penetration_y
	EndIf
	
EndFunc

Func phys_ResponseCollisionPolygons( $polygon_id_1, $polygon_id_2, $point_x, $point_y, $normal_x, $normal_y )
	
	; vector from position to point of collision 	
	Local $r1_x = $point_x - $g_Polygons_Position_X[$polygon_id_1] 
    Local $r1_y = $point_y - $g_Polygons_Position_Y[$polygon_id_1]
    Local $r2_x = $point_x - $g_Polygons_Position_X[$polygon_id_2] 
    Local $r2_y = $point_y - $g_Polygons_Position_Y[$polygon_id_2] 
	
	; velocity on point of collision
    Local $v_1p_x  = $g_Polygons_Velocity_Linear_X[$polygon_id_1] - $g_Polygons_Velocity_Angular[$polygon_id_1]*$r1_y
    Local $v_1p_y  = $g_Polygons_Velocity_Linear_Y[$polygon_id_1] + $g_Polygons_Velocity_Angular[$polygon_id_1]*$r1_x 
    Local $v_2p_x  = $g_Polygons_Velocity_Linear_X[$polygon_id_2] - $g_Polygons_Velocity_Angular[$polygon_id_2]*$r2_y 
    Local $v_2p_y  = $g_Polygons_Velocity_Linear_Y[$polygon_id_2] + $g_Polygons_Velocity_Angular[$polygon_id_2]*$r2_x 

    Local $v_12 = ($v_1p_x - $v_2p_x) * $normal_x + ($v_1p_y - $v_2p_y) * $normal_y
	
	If $v_12 < 0 Then ; polygon move towarts -> collison / dis vielecke bewegen sich im kollisionspunkt aufeinander zu -> kollision
		Local $ja, $jb, $jc, $jd, $je
		
        $ja = (-(1+($g_Polygons_Collision_Frequency[$polygon_id_1] + $g_Polygons_Collision_Frequency[$polygon_id_2])/2) * $v_12)
        $jb = ($normal_x*$normal_x + $normal_y*$normal_y)*($normal_x*$normal_x + $normal_y*$normal_y)

        If $g_Polygons_Mass[$polygon_id_1] <> 0 And $g_Polygons_Mass[$polygon_id_2] <> 0 Then
            $jc = (1/$g_Polygons_Mass[$polygon_id_1] + 1/$g_Polygons_Mass[$polygon_id_2])
        ElseIf $g_Polygons_Mass[$polygon_id_1] = 0 And $g_Polygons_Mass[$polygon_id_2] = 0 Then
            $jc = 0
        ElseIf $g_Polygons_Mass[$polygon_id_1] = 0 Then
            $jc = 1/$g_Polygons_Mass[$polygon_id_2]
        Else
            $jc = 1/$g_Polygons_Mass[$polygon_id_1]
        EndIf

        If $g_Polygons_Tensor[$polygon_id_1] <> 0 Then
            $jd = ($r1_x*$normal_y - $r1_y*$normal_x) * ($r1_x*$normal_y - $r1_y*$normal_x)/$g_Polygons_Tensor[$polygon_id_1]
        Else
            $jd = 0
        EndIf

        If $g_Polygons_Tensor[$polygon_id_2] <> 0 Then
            $je = ($r2_x*$normal_y - $r2_y*$normal_x) * ($r2_x*$normal_y - $r2_y*$normal_x)/$g_Polygons_Tensor[$polygon_id_2]
        Else
            $je = 0
        EndIf

        Local $j = $ja / ($jb*$jc + $jd + $je) * $g_ImpulseScale
	
	    If $g_Polygons_Mass[$polygon_id_1] <> 0 Then
            $g_Polygons_Velocity_Linear_X[$polygon_id_1] += $j / $g_Polygons_Mass[$polygon_id_1] * $normal_x
            $g_Polygons_Velocity_Linear_Y[$polygon_id_1] += $j / $g_Polygons_Mass[$polygon_id_1] * $normal_y
        EndIf

        If $g_Polygons_Mass[$polygon_id_2] <> 0 Then
            $g_Polygons_Velocity_Linear_X[$polygon_id_2] -= $j / $g_Polygons_Mass[$polygon_id_2] * $normal_x
            $g_Polygons_Velocity_Linear_Y[$polygon_id_2] -= $j / $g_Polygons_Mass[$polygon_id_2] * $normal_y
        EndIf

        If $g_Polygons_Tensor[$polygon_id_1] <> 0 Then $g_Polygons_Velocity_Angular[$polygon_id_1] += ($r1_x* $j*$normal_y - $r1_y* $j*$normal_x) / $g_Polygons_Tensor[$polygon_id_1]
        If $g_Polygons_Tensor[$polygon_id_2] <> 0 Then $g_Polygons_Velocity_Angular[$polygon_id_2] += ($r2_x*-$j*$normal_y - $r2_y*-$j*$normal_x) / $g_Polygons_Tensor[$polygon_id_2]

    EndIf	
EndFunc

Func phys_ResponseCollisionPolygonCircle( $polygon_id, $circle_id, $point_x, $point_y, $normal_x, $normal_y )
	
	; vector from position to point of collision 	
	Local $r1_x = $point_x - $g_Polygons_Position_X[$polygon_id] 
    Local $r1_y = $point_y - $g_Polygons_Position_Y[$polygon_id]
    Local $r2_x = $point_x - $g_Circles_Position_X[$circle_id] 
    Local $r2_y = $point_y - $g_Circles_Position_Y[$circle_id] 
	
	; velocity on point of collision
    Local $v_1p_x  = $g_Polygons_Velocity_Linear_X[$polygon_id] - $g_Polygons_Velocity_Angular[$polygon_id]*$r1_y
    Local $v_1p_y  = $g_Polygons_Velocity_Linear_Y[$polygon_id] + $g_Polygons_Velocity_Angular[$polygon_id]*$r1_x 
    Local $v_2p_x  = $g_Circles_Velocity_Linear_X[$circle_id] - $g_Circles_Velocity_Angular[$circle_id]*$r2_y 
    Local $v_2p_y  = $g_Circles_Velocity_Linear_Y[$circle_id] + $g_Circles_Velocity_Angular[$circle_id]*$r2_x 

    Local $v_12 = ($v_1p_x - $v_2p_x) * $normal_x + ($v_1p_y - $v_2p_y) * $normal_y
	 
	If $v_12 < 0 Then ; polygon move towarts -> collison / dis vielecke bewegen sich im kollisionspunkt aufeinander zu -> kollision
		Local $ja, $jb, $jc, $jd, $je
		
        $ja = (-(1+($g_Polygons_Collision_Frequency[$polygon_id] + $g_Circles_Collision_Frequency[$circle_id])/2) * $v_12)
        $jb = ($normal_x*$normal_x + $normal_y*$normal_y)*($normal_x*$normal_x + $normal_y*$normal_y)

        If $g_Polygons_Mass[$polygon_id] <> 0 And $g_Circles_Mass[$circle_id] <> 0 Then
            $jc = (1/$g_Polygons_Mass[$polygon_id] + 1/$g_Circles_Mass[$circle_id])
        ElseIf $g_Polygons_Mass[$polygon_id] = 0 And $g_Circles_Mass[$circle_id] = 0 Then
            $jc = 0
        ElseIf $g_Polygons_Mass[$polygon_id] = 0 Then
            $jc = 1/$g_Circles_Mass[$circle_id]
        Else
            $jc = 1/$g_Polygons_Mass[$polygon_id]
        EndIf

        If $g_Polygons_Tensor[$polygon_id] <> 0 Then
            $jd = ($r1_x*$normal_y - $r1_y*$normal_x) * ($r1_x*$normal_y - $r1_y*$normal_x)/$g_Polygons_Tensor[$polygon_id]
        Else
            $jd = 0
        EndIf

        If $g_Circles_Tensor[$circle_id] <> 0 Then
            $je = ($r2_x*$normal_y - $r2_y*$normal_x) * ($r2_x*$normal_y - $r2_y*$normal_x)/$g_Circles_Tensor[$circle_id]
        Else
            $je = 0
        EndIf

        Local $j = $ja / ($jb*$jc + $jd + $je) * $g_ImpulseScale
	
	    If $g_Polygons_Mass[$polygon_id] <> 0 Then
            $g_Polygons_Velocity_Linear_X[$polygon_id] += $j / $g_Polygons_Mass[$polygon_id] * $normal_x
            $g_Polygons_Velocity_Linear_Y[$polygon_id] += $j / $g_Polygons_Mass[$polygon_id] * $normal_y
        EndIf

        If $g_Circles_Mass[$circle_id] <> 0 Then
            $g_Circles_Velocity_Linear_X[$circle_id] -= $j / $g_Circles_Mass[$circle_id] * $normal_x
            $g_Circles_Velocity_Linear_Y[$circle_id] -= $j / $g_Circles_Mass[$circle_id] * $normal_y
        EndIf

        If $g_Polygons_Tensor[$polygon_id] <> 0 Then $g_Polygons_Velocity_Angular[$polygon_id] += ($r1_x* $j*$normal_y - $r1_y* $j*$normal_x) / $g_Polygons_Tensor[$polygon_id]
        If $g_Circles_Tensor[$circle_id] <> 0 Then $g_Circles_Velocity_Angular[$circle_id] += ($r2_x*-$j*$normal_y - $r2_y*-$j*$normal_x) / $g_Circles_Tensor[$circle_id]

    EndIf	
EndFunc

Func phys_ResponseCollisionCircles( $circle_id_1, $circle_id_2, $point_x, $point_y, $normal_x, $normal_y )
	
	; vector from position to point of collision 	
	Local $r1_x = $point_x - $g_Circles_Position_X[$circle_id_1] 
    Local $r1_y = $point_y - $g_Circles_Position_Y[$circle_id_1]
    Local $r2_x = $point_x - $g_Circles_Position_X[$circle_id_2] 
    Local $r2_y = $point_y - $g_Circles_Position_Y[$circle_id_2] 
	
	; velocity on point of collision
    Local $v_1p_x  = $g_Circles_Velocity_Linear_X[$circle_id_1] - $g_Circles_Velocity_Angular[$circle_id_1]*$r1_y
    Local $v_1p_y  = $g_Circles_Velocity_Linear_Y[$circle_id_1] + $g_Circles_Velocity_Angular[$circle_id_1]*$r1_x 
    Local $v_2p_x  = $g_Circles_Velocity_Linear_X[$circle_id_2] - $g_Circles_Velocity_Angular[$circle_id_2]*$r2_y 
    Local $v_2p_y  = $g_Circles_Velocity_Linear_Y[$circle_id_2] + $g_Circles_Velocity_Angular[$circle_id_2]*$r2_x 

    Local $v_12 = ($v_1p_x - $v_2p_x) * $normal_x + ($v_1p_y - $v_2p_y) * $normal_y

	If $v_12 < 0 Then 
		Local $ja, $jb, $jc, $jd, $je
		
        $ja = (-(1+($g_Circles_Collision_Frequency[$circle_id_1] + $g_Circles_Collision_Frequency[$circle_id_2])/2) * $v_12)
        $jb = ($normal_x*$normal_x + $normal_y*$normal_y)*($normal_x*$normal_x + $normal_y*$normal_y)

        If $g_Circles_Mass[$circle_id_1] <> 0 And $g_Circles_Mass[$circle_id_2] <> 0 Then
            $jc = (1/$g_Circles_Mass[$circle_id_1] + 1/$g_Circles_Mass[$circle_id_2])
        ElseIf $g_Circles_Mass[$circle_id_1] = 0 And $g_Circles_Mass[$circle_id_2] = 0 Then
            $jc = 0
        ElseIf $g_Polygons_Mass[$circle_id_1] = 0 Then
            $jc = 1/$g_Circles_Mass[$circle_id_2]
        Else
            $jc = 1/$g_Circles_Mass[$circle_id_1]
        EndIf

        If $g_Circles_Tensor[$circle_id_1] <> 0 Then
            $jd = ($r1_x*$normal_y - $r1_y*$normal_x) * ($r1_x*$normal_y - $r1_y*$normal_x)/$g_Circles_Tensor[$circle_id_1]
        Else
            $jd = 0
        EndIf

        If $g_Circles_Tensor[$circle_id_2] <> 0 Then
            $je = ($r2_x*$normal_y - $r2_y*$normal_x) * ($r2_x*$normal_y - $r2_y*$normal_x)/$g_Circles_Tensor[$circle_id_2]
        Else
            $je = 0
        EndIf

        Local $j = $ja / ($jb*$jc + $jd + $je) * $g_ImpulseScale

        If $g_Circles_Mass[$circle_id_1] <> 0 Then
            $g_Circles_Velocity_Linear_X[$circle_id_1] += $j / $g_Circles_Mass[$circle_id_1] * $normal_x
            $g_Circles_Velocity_Linear_Y[$circle_id_1] += $j / $g_Circles_Mass[$circle_id_1] * $normal_y
        EndIf

        If $g_Circles_Mass[$circle_id_2] <> 0 Then
            $g_Circles_Velocity_Linear_X[$circle_id_2] -= $j / $g_Circles_Mass[$circle_id_2] * $normal_x
            $g_Circles_Velocity_Linear_Y[$circle_id_2] -= $j / $g_Circles_Mass[$circle_id_2] * $normal_y
        EndIf

        If $g_Circles_Tensor[$circle_id_1] <> 0 Then $g_Circles_Velocity_Angular[$circle_id_1] += ($r1_x* $j*$normal_y - $r1_y* $j*$normal_x) / $g_Circles_Tensor[$circle_id_1]
        If $g_Circles_Tensor[$circle_id_2] <> 0 Then $g_Circles_Velocity_Angular[$circle_id_2] += ($r2_x*-$j*$normal_y - $r2_y*-$j*$normal_x) / $g_Circles_Tensor[$circle_id_2]

    EndIf	
EndFunc

Func phys_ResponseFrictionPolygon( $polygon_id, $point_x, $point_y, $normal_x, $normal_y )
	Local $Dot = phys_vec_dot( $normal_x, $g_Polygons_Force_Linear_X[$polygon_id], $normal_y, $g_Polygons_Force_Linear_Y[$polygon_id] )
	
	If $Dot < 0 Then ; force is in direction of surface
		Local $ForceProjectedSurfaceX = $normal_y * $g_Polygons_Force_Linear_X[$polygon_id]
		Local $ForceProjectedSurfaceY = -$normal_x * $g_Polygons_Force_Linear_Y[$polygon_id]
	
		phys_PolygonAddForcePoint( $polygon_id, $point_x, $point_y, $ForceProjectedSurfaceX, $ForceProjectedSurfaceY )
	EndIf
EndFunc

Func phys_ResponseFrictionCircle( $circle_id, $point_x, $point_y, $normal_x, $normal_y )
	Local $Dot = phys_vec_dot( $normal_x, $g_Circles_Force_Linear_X[$circle_id], $normal_y, $g_Circles_Force_Linear_Y[$circle_id] )
	
	If $Dot < 0 Then ; force is in direction of surface
		Local $ForceProjectedSurfaceX = $normal_y * $g_Circles_Force_Linear_X[$circle_id]
		Local $ForceProjectedSurfaceY = -$normal_x * $g_Circles_Force_Linear_Y[$circle_id]
	
		phys_CircleAddForcePoint( $circle_id, $point_x, $point_y, $ForceProjectedSurfaceX, $ForceProjectedSurfaceY )
	EndIf
EndFunc