;PhysicEngine_3_Math.au3

Global Const $PI = 4 * ATan(1)
Global Const $TWO_PI = 2 * $PI
Global Const $HALF_PI = 0.5 * $PI
Global Const $RAD_TO_DEG = 180 / $PI
Global Const $DEG_TO_RAD = $PI / 180

Func phys_LinesIntersect( $line_1_start_x, $line_1_start_y, $line_1_end_x, $line_1_end_y, $line_2_start_x, $line_2_start_y, $line_2_end_x, $line_2_end_y, ByRef $IntersectionPoint )
	
	Local $commonDenominator = ($line_2_end_y - $line_2_start_y) * ($line_1_end_x - $line_1_start_x) - ($line_2_end_x - $line_2_start_x) * ($line_1_end_y - $line_1_start_y)
	
	Local $numeratorA = ($line_2_end_x - $line_2_start_x) * ($line_1_start_y - $line_2_start_y) - ($line_2_end_y - $line_2_start_y) * ($line_1_start_x - $line_2_start_x)
	
	Local $numeratorB = ($line_1_end_x - $line_1_start_x) * ($line_1_start_y - $line_2_start_y) - ($line_1_end_y - $line_1_start_y) * ($line_1_start_x - $line_2_start_x)

	If $commonDenominator = 0 Then
		If ($numeratorA = 0) And ($numeratorB) Then
			If phys_vec_equal( $line_1_start_x, $line_1_start_y, $line_2_start_x, $line_2_start_y ) Or phys_vec_equal( $line_1_start_x, $line_1_start_y, $line_2_end_x, $line_2_end_y ) Then
				$IntersectionPoint[0] = $line_1_start_x
				$IntersectionPoint[1] = $line_1_start_y
			ElseIf phys_vec_equal( $line_1_end_x, $line_1_end_y, $line_2_end_x, $line_2_end_y ) Or phys_vec_equal( $line_1_end_x, $line_1_end_y, $line_2_start_x, $line_2_start_y ) Then
				$IntersectionPoint[0] = $line_1_end_x
				$IntersectionPoint[1] = $line_1_end_y
			Else
				$IntersectionPoint[0] = 0.25 * ($line_1_start_x + $line_1_end_x + $line_2_start_x + $line_2_end_x)
				$IntersectionPoint[1] = 0.25 * ($line_1_start_y + $line_1_end_y + $line_2_start_y + $line_2_end_y)
			EndIf
			
			Return True
		Else
			Return False
		EndIf
	EndIf
	
	Local $divAD = $numeratorA / $commonDenominator
	
	If ($divAD < 0) Or ($divAD > 1) Then Return False
	
	Local $divBD = $numeratorB / $commonDenominator
	
	If ($divBD < 0) Or ($divBD > 1) Then Return False
	
	$IntersectionPoint[0] = $line_1_start_x + $divAD * ($line_1_end_x - $line_1_start_x)
	$IntersectionPoint[1] = $line_1_start_y + $divAD * ($line_1_end_y - $line_1_start_y)
	
	Return True
EndFunc

Func phys_LineNearestPoint( $line_1_x, $line_1_y, $line_2_x, $line_2_y, $point_x, $point_y )

	Local $LinePointX = $point_x - $line_1_x
	Local $LinePointY = $point_y - $line_1_y
	
	Local $LineVectorX = $line_2_x - $line_1_x
	Local $LineVectorY = $line_2_y - $line_1_y

	Local $LineLength = phys_vec_length( $LineVectorX, $LineVectorY )
	
	$LineVectorX /= $LineLength
	$LineVectorY /= $LineLength
	
	Local $Lambda = phys_vec_dot( $LineVectorX, $LineVectorY, $LinePointX, $LinePointY )
	
	Dim $Point[2]
	
	If $Lambda < 0 Then 
		$Point[0] = $line_1_x
		$Point[1] = $line_1_y
	ElseIf $Lambda > $LineLength Then 
		$Point[0] = $line_2_x
		$Point[1] = $line_2_y
	Else
		$Point[0] = $line_1_x + $LineVectorX * $Lambda
		$Point[1] = $line_1_y + $LineVectorY * $Lambda
	EndIf
	
	Return $Point
EndFunc

Func phys_RayNearestPoint( $ray_orgin_x, $ray_orgin_y, $ray_direction_x, $ray_direction_y, $point_x, $point_y )

	Local $LinePointX = $point_x - $ray_orgin_x
	Local $LinePointY = $point_y - $ray_orgin_y
	
	phys_vec_normalize( $ray_direction_x, $ray_direction_y )
	
	Local $Lambda = phys_vec_dot( $ray_direction_x, $ray_direction_y, $LinePointX, $LinePointY )
	
	Dim $Point[2]
	
	$Point[0] = $ray_orgin_x + $ray_direction_x * $Lambda
	$Point[1] = $ray_orgin_y + $ray_direction_y * $Lambda

	Return $Point
EndFunc

Func phys_LineOrthogonal( $point_1_x, $point_1_y, $point_2_x, $point_2_y, ByRef $orthogonal_x, ByRef $orthogonal_y )
	Local $DeltaX = $point_2_x - $point_1_x
	Local $DeltaY = $point_2_y - $point_1_y
	
	phys_vec_normalize( $DeltaX, $DeltaY )
	
	$orthogonal_x = $DeltaY
	$orthogonal_y = -$DeltaX
EndFunc

Func phys_LineCircleIntersection( $line_1_x, $line_1_y, $line_2_x, $line_2_y, $center_x, $center_y, $radius, ByRef $num_points, ByRef $points )
	
	$num_points = 0
	
	Local $LinePointX = $center_x - $line_1_x
	Local $LinePointY = $center_y - $line_1_y
	
	Local $LineVectorX = $line_2_x - $line_1_x
	Local $LineVectorY = $line_2_y - $line_1_y

	Local $LineLength = phys_vec_length( $LineVectorX, $LineVectorY )
	
	$LineVectorX /= $LineLength
	$LineVectorY /= $LineLength
	
	Local $Lambda = phys_vec_dot( $LineVectorX, $LineVectorY, $LinePointX, $LinePointY )
	
	Local $NearestX = $line_1_x + $LineVectorX * $Lambda
	Local $NearestY = $line_1_y + $LineVectorY * $Lambda
		
	Local $Distance = phys_vec_distance( $NearestX, $NearestY, $center_x, $center_y )
	
	If $Distance > $radius Then Return False
	
	Local $HalfCathetus = Sqrt($radius*$radius - $Distance*$Distance)  ; a+b = c => a = sqrt( c-b )
	
	Local $Lambda1 = $Lambda + $HalfCathetus

	If ($Lambda1 >= 0) And ($Lambda1 <= $LineLength) Then 
		$points[0] = $line_1_x + $LineVectorX * $Lambda1
		$points[1] = $line_1_y + $LineVectorY * $Lambda1
		$num_points = 1
	EndIf
	
	Local $Lambda2 = $Lambda - $HalfCathetus
	
	If ($Lambda2 >= 0) And ($Lambda2 <= $LineLength) Then 
		$points[$num_points*2  ] = $line_1_x + $LineVectorX * $Lambda2
		$points[$num_points*2+1] = $line_1_y + $LineVectorY * $Lambda2
		$num_points += 1
	EndIf
	
	Return ($num_points <> 0)
EndFunc

Func phys_PointInCircle( $point_x, $point_y, $center_x, $center_y, $radius )
	
	Local $DeltaX = $center_x - $point_x
	Local $DeltaY = $center_y - $point_y
	
	Return ($radius >= Sqrt( $DeltaX * $DeltaX + $DeltaY * $DeltaY ))
EndFunc

Func phys_vec_distance( $v_1_x, $v_1_y, $v_2_x, $v_2_y )
	Local $DeltaX = $v_2_x - $v_1_x
	Local $DeltaY = $v_2_y - $v_1_y
	
	Return Sqrt( $DeltaX * $DeltaX + $DeltaY * $DeltaY )
EndFunc

Func phys_vec_dot( $v_1_x, $v_1_y, $v_2_x, $v_2_y )
	Return $v_1_x * $v_2_x + $v_1_y * $v_2_y
EndFunc

Func phys_vec_length( $v_x, $v_y )
	Return Sqrt( $v_x * $v_x + $v_y * $v_y )
EndFunc

Func phys_vec_equal( $v_1_x, $v_1_y, $v_2_x, $v_2_y )
	Return (($v_1_x = $v_2_x) And ($v_1_y = $v_2_y))
EndFunc

Func phys_vec_iszero( $v_x, $v_y )
	Return (($v_x = 0) And ($v_y = 0))
EndFunc

Func phys_vec_normalize( ByRef $v_x, ByRef $v_y )
	Local $Length = phys_vec_length( $v_x, $v_y )
	
	$v_x /= $Length
	$v_y /= $Length
EndFunc

Func phys_cross2( $v_1_x, $v_1_y, $v_2_x, $v_2_y )
	Return $v_1_x * $v_2_y - $v_1_y * $v_2_x
EndFunc

Func phys_vec_setlength( ByRef $v_x, ByRef $v_y, $new_length )
	Local $Length = phys_vec_length( $v_x, $v_y ) / $new_length
	
	$v_x /= $Length
	$v_y /= $Length
EndFunc

Func phys_vec_project( $v_x, $v_y, ByRef $v_project_x, ByRef $v_project_y )
	phys_vec_normalize( $v_x, $v_y )
	
	Local $Lambda = phys_vec_dot( $v_x, $v_y, $v_project_x, $v_project_y )
	
	$v_project_x = $v_x * $Lambda
	$v_project_y = $v_y * $Lambda	
EndFunc